#!python
# -*- coding: utf-8 -*-
__version__ = "$Revision: 1.11 $"
# $Source: /home/mechanoid/projects/py/theano/log-regr/RCS/main.py,v $
#
#       OS : GNU/Linux 4.7.6-1-ARCH 
# COMPILER : Python 3.5.2
#
#   AUTHOR : Evgeny S. Borisov
# 
#    http://www.mechanoid.kiev.ua
#  e-mail : nn@mechanoid.kiev.ua
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

import numpy as np
import pandas as pd
import theano
import theano.tensor as T
import matplotlib.pyplot as plt

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
df = pd.read_csv('data/points_g.txt',sep=',') # загружаем данные класса 1
X1 = np.asarray(df,dtype=np.float32) # конвернтируем в матрицу
df = pd.read_csv('data/points_r.txt',sep=',') # загружаем данные класса 2
X2 = np.asarray(df,dtype=np.float32) # конвернтируем в матрицу
del df 

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

# размеры классов
sz1=X1.shape[0]
sz2=X2.shape[0]
feats = X1.shape[1] # размер входного вектора

# собираем данные в одну таблицу
X = np.vstack((X1,X2)).astype(theano.config.floatX)

# список правильных ответов
Y = np.hstack( ( np.zeros(sz1),np.ones(sz2) ) ).astype(theano.config.floatX)

del X1,X2

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

# разделяем набор на тестовый и учебный
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split( X, Y, test_size=0.3, random_state=0)

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

# определяем символические переменные (Theano symbolic variables)
x = T.matrix("x") # вход классификатора
y = T.vector("y") # результат

rng = np.random # генератор случайных значений
# определяем и инициализируем глобальные (общие) переменные 
# для использования функциями обучения (train) и прогнозирования (predict) 
w = theano.shared(rng.randn(feats).astype(theano.config.floatX), name="w") # вектор весов
b = theano.shared(np.asarray(0., dtype=theano.config.floatX), name="b") # сдвиг

# определяем выражения (Theano expression graph)

# классификатор, вероятность принадлежности к классу
p_1 = 1.0 / (1.0 + T.exp(-T.dot(x, w)-b)) 
prediction = p_1 > 0.5 # прогнозирование, ответ классификатора
xent = -y*T.log(p_1) - (1-y)*T.log(1-p_1) # функция ошибки (cross-entropy)
# функция стоимости (с регуляризацией), общая ошибка для всех точек набора
cost = xent.mean() + 0.01*(w**2).sum() 
gw,gb = T.grad(cost, [w,b]) # градиент функции стоимости

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

learn_rate = 0.05 # скорость обучения

# компилируем функцию обучения
train = theano.function(
    inputs=[x,y], # вход, учебный набор
    outputs=cost, # выход, текущее значение функции стоимости
    # изменение параметров, в соответствии с методом градиентного спуска
    updates=[(w, w-learn_rate*gw), (b, b-learn_rate*gb)], 
    name = "train")

# компилируем функцию прогнозирования
predict = theano.function(
    inputs=[x], # вход, набор точек для оценки 
    outputs=prediction, # выход, номер класса (0 или 1)
    name = "predict")



# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# print(theano.pp(p_1)) # expression pretty printing
# theano.printing.debugprint(predict) # post-compilant debug print
theano.printing.pydotprint(predict, outfile="result/predict.png", var_with_name_simple=True) # picture plot graphs

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

training_steps = int(1e5) # количество циклов обучения
cost_min = 0.07 # порог ошибки

for i in range(training_steps): # цикл обучения
    err = train(X_train, y_train) # вычисление градиента и изменение весов
    if not(i % 1000): # печатаем значение ошибки каждые 1000 шагов
        print("cost[",i,"]:",err,end='\r')
    if err < cost_min:
        break
print("cost[",i,"]:",err,end='\r')

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

y_res = predict(X_test) # тестируем классификатор



f, axarr = plt.subplots(1,3, sharey=True,figsize=(17,5))

# рисуем учебный набор
axarr[0].grid(True)
axarr[0].set_xlim([0.0,21.0])
axarr[0].set_ylim([-0.5,14.0])
axarr[0].set_title('train data')
axarr[0].scatter(x=X_train[y_train==1,0], y=X_train[y_train==1,1], alpha=0.8, c='blue')
axarr[0].scatter(x=X_train[y_train==0,0], y=X_train[y_train==0,1], alpha=0.8, c='red')
#axarr[0].set_aspect(1.0)

# рисуем тестовый набор
axarr[1].grid(True)
axarr[1].set_xlim([0.0,21.0])
axarr[1].set_ylim([-0.5,14.0])
axarr[1].set_title('test data')
axarr[1].scatter(x=X_test[y_test==1,0], y=X_test[y_test==1,1], alpha=0.8, c='blue')
axarr[1].scatter(x=X_test[y_test==0,0], y=X_test[y_test==0,1], alpha=0.8, c='red')

# рисуем результаты теста
axarr[2].grid(True)
axarr[2].set_xlim([0.0,21.0])
axarr[2].set_ylim([-0.5,14.0])
axarr[2].set_title('result')
# правильные ответы
axarr[2].scatter(x=X_test[(y_res == 0), 0], y=X_test[(y_res == 0), 1], alpha=0.8, c='blue')
axarr[2].scatter(x=X_test[(y_res == 1), 0], y=X_test[(y_res == 1), 1], alpha=0.8, c='red')
# ошибки теста
axarr[2].scatter(x=X_test[(y_res != y_test), 0], y=X_test[(y_res != y_test), 1], alpha=0.8, c='yellow',label='errors')
axarr[2].legend(loc="upper left")

plt.savefig('result/plot1.png',dpi=60)
